<?php

/**
 * Discussion Board Follow class
 * @since 1.2.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class CT_DB_Pro_Follow {

	public function __construct() {
		//
	}

	/**
	 * Initialize the class and start calling our hooks and filters
	 * @since 1.2.0
	 */

	public function init() {
		add_filter( 'the_content', array( $this, 'filter_single_content' ) );

		add_filter( 'ctdb_followers_count', array( $this, 'add_followers_box' ) );
		add_filter( 'ctdb_info_meta_close_classic_meta', array( $this, 'follow_button' ) );

		add_action( 'wp_ajax_update_follow_status', array( $this, 'update_follow_status' ) );

		add_action( 'comment_post', array( $this, 'notify_followers' ), 10, 3 );
		add_action( 'transition_comment_status', array( $this, 'notify_followers_after_moderation' ), 10, 3 );

		// Added 1.6.2
		add_action( 'transition_post_status', array( $this, 'email_notification' ), 10, 3 );
	}

	/**
	 * Return markup for followers box on standard layout
	 * @since 1.2.0
	 */
	public function filter_single_content( $content ) {
		// Check if user is logged in
		if( ! is_user_logged_in() ) {
			return $content;
		}
		/**
		 * Check the layout.
		 * Only display followers here if the layout is standard
		 * @since 1.3.0
		 */
		$design_options = get_option( 'ctdb_design_settings' );
		if( isset( $design_options['info_bar_layout'] ) && $design_options['info_bar_layout'] != 'standard' ) {
			return $content;
		}
		// If we're on a single page for the discussion-topic post type and we don't have access
		if( is_single() && 'discussion-topics' == get_post_type() && is_main_query() ) {
			$user_id = get_current_user_id();

			// Get user ID
			// Check if user is subscribed to this topic
			$postid = get_the_ID();

		//	update_post_meta( $postid, 'ctdb_follow_this_topic', array() );
			$followers = get_post_meta( $postid, 'ctdb_follow_this_topic', true );
			$followers = apply_filters( 'ctdb_follow_this_topic', $followers, $postid );
			$checked = '';
			if( ! empty( $followers ) ) {
				// See if the user is already subscribed
				if( in_array( $user_id, $followers ) ) {
					$checked = 'checked="checked"';
				}
				$followers_count = count( $followers );
			} else {
				// Array is empty
				$followers_count = 0;
			}

			// Get the message
			$message = ctdb_get_follow_message();
			$content .= wp_nonce_field( 'follow_update_nonce', 'follow_update_nonce' );
			$content .= '<div class="ctdb-follow-box">';
			$content .= '<input type="checkbox" class="ctdb-follow-checkbox" id="ctdb-follow-checkbox" name="ctdb-follow-checkbox" ' . $checked . '>';
			$content .= '<label for="ctdb-follow-checkbox">' . wp_kses_post( $message ) . '</label>';
			if( isset( $options['display_follower_numbers'] ) ) {
				$content .= '<p class="ctdb-follower-count"><span>';
				$text_after_followers = ctdb_get_text_after_followers();
				$content .= $followers_count . '</span> ' . esc_html( $text_after_followers );
				$content .= '</p>';
			}
			$content .= '</div>';

		}

		return $content;
	}

	/**
	 * Return markup for followers button
	 * @since 1.2.0
	 */
	public function follow_button( $content ) {

		// Check if user is logged in
		if( ! is_user_logged_in() ) {
			return $content;
		}
		/**
		 * Check the layout.
		 * Only display followers here if the layout is standard
		 * @since 1.3.0
		 */
		$design_options = get_option( 'ctdb_design_settings' );
		if( isset( $design_options['info_bar_layout'] ) && $design_options['info_bar_layout'] != 'classic' ) {
			return $content;
		}
		// If we're on a single page for the discussion-topic post type and we don't have access
		if( is_single() && 'discussion-topics' == get_post_type() && is_main_query() ) {
			$user_id = get_current_user_id();

			// Get user ID
			// Check if user is subscribed to this topic
			$postid = get_the_ID();

		//	update_post_meta( $postid, 'ctdb_follow_this_topic', array() );
			$followers = get_post_meta( $postid, 'ctdb_follow_this_topic', true );
			$followers = apply_filters( 'ctdb_follow_this_topic', $followers, $postid );
			$checked = '';
			$class = 'ctdb-not-following';
			if( ! empty( $followers ) ) {
				// See if the user is already subscribed
				if( in_array( $user_id, $followers ) ) {
					$class = 'ctdb-is-following';
				}
				$followers_count = count( $followers );
			} else {
				// Array is empty
				$followers_count = 0;
			}
			// Use icons?
			$options = get_option( 'ctdb_design_settings' );
			$show_icons = isset( $options['enqueue_dashicons'] );

			// Get the message

			$follow_message = ctdb_get_button_follow_text();
			$unfollow_message = ctdb_get_button_unfollow_text();

			$content .= wp_nonce_field( 'follow_update_nonce', 'follow_update_nonce' );
			$content .= '<button class="ctdb-follow-button ' . esc_attr( $class ) . '">';
			if( $show_icons ) $content .= '<span class="dashicons dashicons-visibility"></span><span class="dashicons dashicons-hidden"></span> ';
			$content .= '<span class="follow">' . strip_tags( $follow_message ) . '</span>';
			$content .= '<span class="unfollow">' . strip_tags( $unfollow_message ) . '</span>';
			$content .= '</button>';

		}

		return $content;

	}

	/**
	 * Display follower count in table view
	 * @since 1.4.0
	 */
	public function add_followers_box( $followers_count ) {

		// Check if user is logged in
		if( ! is_user_logged_in() ) {
			return $followers_count;
		}

		/**
		 * Check the layout.
		 * Only display followers here if the layout is standard
		 * @since 1.3.0
		 */
		$design_options = get_option( 'ctdb_design_settings' );
		if( ! isset( $design_options['info_bar_layout'] ) || $design_options['info_bar_layout'] != 'table' ) {
			return $followers_count;
		}

		$output = '';

		$user_id = get_current_user_id();

		// Get user ID
		// Check if user is subscribed to this topic
		$postid = get_the_ID();

		$followers = get_post_meta( $postid, 'ctdb_follow_this_topic', true );
		$followers = apply_filters( 'ctdb_follow_this_topic', $followers, $postid );
		$checked = '';
		if( ! empty( $followers ) ) {
			// See if the user is already subscribed
			if( in_array( $user_id, $followers ) ) {
				$checked = 'checked="checked"';
			}
			$followers_count = count( $followers );
		} else {
			// Array is empty
			$followers_count = 0;
		}

		// If we're on a single page for the discussion-topic post type and we don't have access
		if( is_single() && 'discussion-topics' == get_post_type() ) {

			// Get the message

			$options = get_option( 'ctdb_followers_settings' );
			$message = ctdb_get_follow_message();
			$output = wp_nonce_field( 'follow_update_nonce', 'follow_update_nonce' );

			$output .= '<div class="ctdb-follower-count"><span>' . $followers_count . '</span> ';
			$output .= '&nbsp;&nbsp;<input type="checkbox" class="ctdb-follow-checkbox" id="ctdb-follow-checkbox" name="ctdb-follow-checkbox" ' . $checked . '>';
			$output .= '<label for="ctdb-follow-checkbox">' . stripslashes( strip_tags( wp_kses_post( $message ) ) ) . '</label></div>';

		} else if( is_archive() && 'discussion-topics' == get_post_type() ) {
			// On an archive page, just return the number of followers
			$output .= $followers_count;
		}

		return $output;
	}

	/**
	 *
	 * @since 1.0.0
	 */
	public function update_follow_status() {
		check_ajax_referer( 'follow_update_nonce', 'security' );
		$return = 'fail';
		if ( ! empty( $_REQUEST['userid'] ) && ! empty( $_REQUEST['postid'] ) ) {
			$userid = absint( $_REQUEST['userid'] );
			$postid = absint( $_REQUEST['postid'] );
			$checked = stripslashes( strip_tags( $_REQUEST['checked'] ) );

			// We need to update the post meta field
			$followers = get_post_meta( $postid, 'ctdb_follow_this_topic', true );
			$followers = apply_filters( 'ctdb_follow_this_topic', $followers, $postid );
			if( $checked == 'true' ) {
				// Add the user ID
				if( empty ( $followers ) ) {
					$followers = array( $userid );
				} else if( ! in_array( $userid, $followers ) ) {
					// Only add the ID if it's not already there
					$followers[] = $userid;
				}
			} else {
				// Remove the user ID
				if( ( $key = array_search( $userid, $followers ) ) !== false ) {
					unset( $followers[$key] );
				}
			}
			$return = count( $followers );

			update_post_meta( $postid, 'ctdb_follow_this_topic', $followers );

		}
		echo $return;
		die();
	}

	/**
	 * Notify followers when a comment is approved
	 * @since 1.2.0
	 */
	public function notify_followers_after_moderation( $new_status, $old_status, $comment ) {
		if( $new_status != $old_status && $new_status == 'approved' ) {
			$this->notify_followers_new_comment( $comment );
		}
	}

	/**
	 * Notify followers when a comment is approved
	 * @since 1.2.0
	 */
	public function notify_followers( $comment_ID, $comment_approved ) {
		if( $comment_approved === 1 ) {
			$comment = get_comment( $comment_ID );
			$this->notify_followers_new_comment( $comment );
		}
	}

	/**
	 * Notify followers when a new comment is posted
	 * @since 1.2.0
	 */
	public function notify_followers_new_comment( $comment ) {

		global $post;

		// Grab some information about the comment
		$author = $comment->comment_author;
		$comment_post_ID = $comment->comment_post_ID;

		// Get all followers' emails
		$post = get_post( $comment_post_ID );
		$author_id = $post->post_author;
		$post_author = get_the_author_meta( 'user_email', $author_id );

		$followers = get_post_meta( $post->ID, 'ctdb_follow_this_topic', true );
		$followers = apply_filters( 'ctdb_follow_this_topic', $followers, $post->ID );
		$follower_list = array();
		if( ! empty( $followers ) ) {
			foreach( $followers as $follower ) {
				$follower_data = get_userdata( $follower );
				// Check we're not including commenter email
				if( $follower_data->user_email != $comment->comment_author_email ) {
					$follower_list[] = $follower_data->user_email;
				}
			}
		}

		// Also check for people following this board, if enabled
		// @since 1.6.2
		$board_following = ctdb_are_follower_boards_enabled();
		if( $board_following ) {
			$board_id = ctdb_topic_board_id( $comment_post_ID );
			// Exclude commenter email from the follower list
			$board_follower_list = ctdb_get_board_followers( $board_id, $comment->comment_author_email );
			// Merge both lists
			if( is_array( $board_follower_list ) ) {
				$follower_list = array_merge( $follower_list, $board_follower_list );
			}
		}

		if( empty( $follower_list ) ) {
			// If there are no followers, do nothing
			return;
		}

		// Remove any duplicates for neatness
		$follower_list = array_unique( $follower_list );

		// html_entity_decode converts entities to characters
		$title = html_entity_decode( $post->post_title, ENT_QUOTES, 'UTF-8' );

		$subject = get_bloginfo( 'name' ) . ': ' . __( 'New comment on ', 'discussion-board-pro' ) . $title;
		$content = '<p>' . __( 'A new comment has been left on ', 'discussion-board-pro' );
		$content .= $title . '</p>';

		$content .= '<p>' . wp_kses_post( $comment->comment_content ) . '</p>';

		$content .= '<p><a href="' . esc_url( get_permalink( $post->ID ) . '#comment-' . $comment->comment_ID ) . '">' . esc_url( get_permalink( $post->ID ) . '#comment-' . $comment->comment_ID ) . '</a></p>';

		$content = apply_filters( 'ctdb_follower_notification_new_comment_email', $content, $post, $comment );

		// Add a message to the user to remind them why they're getting this email
		$reason_text = __( 'You are receiving this email because you chose to follow this topic. You can stop any further messages by following the link above and clicking the Unfollow button.', 'discussion-board-pro' );
		$reason_text = apply_filters( 'ctdb_follower_notification_reason_message', $reason_text );

		if( ! empty( $reason_text ) ) {
			$content .= '<p>' . wp_kses_post( $reason_text ) . '</p>';
		}

		// Set HTML content type
		add_filter( 'wp_mail_content_type', array( $this, 'set_html_content_type' ) );

		// Send mail
		$headers = array(
			'Bcc:' . join( ',', $follower_list )
		);

		// Send to blank email address and blind copy everyone else
		$to_email = '';
		// Filter this email address if required
		$to_email = apply_filters( 'ctdb_followed_topic_to_email', $to_email );

		wp_mail( $to_email, $subject, $content, $headers );

		// Reset content-type to avoid conflicts -- http://core.trac.wordpress.org/ticket/23578
		remove_filter( 'wp_mail_content_type', array( $this, 'set_html_content_type' ) );

	}

	/**
	 * Send email notification to board followers when topic is created
	 * @since 1.6.2
	 */
	public function email_notification( $new_status, $old_status, $post ) {

		// Only send a mail if the status has changed to publish
		if( $new_status == $old_status || $new_status != 'publish' ) {
			return;
		}

		// Only notify for discussion topics post type
		if( 'discussion-topics' != $post->post_type ) {
			return;
		}

		// Only send notifications for discussion topics post type
		$board_following = ctdb_are_follower_boards_enabled();
		if( ! $board_following ) {
			return;
		}

		// Get board ID
		$board_id = ctdb_topic_board_id( $post->ID );
		// Exclude author email from the follower list
		$author_id = $post->post_author;
		$author_email = get_the_author_meta( 'email', $author_id );
		// Get board followers
		$follower_list = ctdb_get_board_followers( $board_id, $author_email );


		// If we don't have any followers, don't continue
		if( empty( $follower_list ) ) {
			return;
		}

		$message = '';

		// Set HTML content type
		add_filter( 'wp_mail_content_type', array( $this, 'set_html_content_type' ) );

		$to = '';

		$subject = get_bloginfo( 'name' ) . ': ' . __( 'New topic posted on a board you follow', 'discussion-board-pro' );

		$message = '<p>' . __( 'A new topic has been created on a board you follow:', 'discussion-board-pro' ) . '</p>';
		$message .= '<p><strong>' . $post->post_title . '</strong></p>';
		$message .= '<p>' . $post->post_content . '</p>';
		$message .= '<p><a href="' . esc_url( get_permalink( $post->ID ) ) . '">' . esc_url( get_permalink( $post->ID ) ) . '</a></p>';

		$message = apply_filters( 'ctdb_follower_notification_new_topic_email', $message, $post, $board_id );

		// Send mail
		$headers = array(
			'Bcc:' . join( ',', $follower_list )
		);

		// Set HTML content type
		add_filter( 'wp_mail_content_type', array( $this, 'set_html_content_type' ) );

		wp_mail( $to, $subject, $message, $headers );

		// Reset content-type to avoid conflicts -- http://core.trac.wordpress.org/ticket/23578
		remove_filter( 'wp_mail_content_type', array( $this, 'set_html_content_type' ) );

	}


	public function set_html_content_type() {
		return 'text/html';
	}

	/**
	 * Returns the admin email address if alternative address not set
	 * @since 1.2.0
	 */
	public function admin_email() {
		$options = get_option( 'ctdb_options_settings' );
		$notification_email = $options['notification_email'];
		// Set HTML content type
		add_filter( 'wp_mail_content_type', array( $this, 'set_html_content_type' ) );
		if( ! empty( $notification_email ) || ! is_email( $notification_email ) ) {
			$to = $notification_email;
		} else {
			$to = get_option( 'admin_email' );
		}
		return $to;
	}

}

$CT_DB_Pro_Follow = new CT_DB_Pro_Follow();
$CT_DB_Pro_Follow->init();
